@@ -1,8 +1,8 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/shapes.py
 
-__version__=''' $Id$ '''
+__version__='3.3.0'
 __doc__='''Core of the graphics library - defines Drawing and Shapes'''
 
 import os, sys
@@ -10,7 +10,7 @@ from math import pi, cos, sin, tan, sqrt
 from pprint import pprint
 
 from reportlab.platypus import Flowable
-from reportlab.rl_config import shapeChecking, verbose, defaultGraphicsFontName as _baseGFontName, _unset_
+from reportlab.rl_config import shapeChecking, verbose, defaultGraphicsFontName as _baseGFontName, _unset_, decimalSymbol
 from reportlab.lib import logger
 from reportlab.lib import colors
 from reportlab.lib.validators import *
@@ -20,6 +20,7 @@ from reportlab.lib.attrmap import *
 from reportlab.lib.rl_accel import fp_str
 from reportlab.pdfbase.pdfmetrics import stringWidth
 from reportlab.lib.fonts import tt2ps
+from reportlab.pdfgen.canvas import FILL_EVEN_ODD, FILL_NON_ZERO
 _baseGFontNameB = tt2ps(_baseGFontName,1,0)
 _baseGFontNameI = tt2ps(_baseGFontName,0,1)
 _baseGFontNameBI = tt2ps(_baseGFontName,1,1)
@@ -55,14 +56,13 @@ STATE_DEFAULTS = {   # sensible defaults for all
     'overprintMask': 0,
 
     'fillColor': colors.black,   #...or text will be invisible
-    #'fillRule': NON_ZERO_WINDING, - these can be done later
+    'fillMode': FILL_EVEN_ODD,      #same as pdfgen.canvas
 
     'fontSize': 10,
     'fontName': _baseGFontName,
     'textAnchor':  'start' # can be start, middle, end, inherited
     }
 
-
 ####################################################################
 # math utilities.  These could probably be moved into lib
 # somewhere.
@@ -120,7 +120,9 @@ def transformPoint(A,v):
     return (A[0]*v[0]+A[2]*v[1]+A[4],A[1]*v[0]+A[3]*v[1]+A[5])
 
 def transformPoints(matrix, V):
-    return list(map(transformPoint, V))
+    r = [transformPoint(matrix,v) for v in V]
+    if isinstance(V,tuple): r = tuple(r)
+    return r
 
 def zTransformPoints(matrix, V):
     return list(map(lambda x,matrix=matrix: zTransformPoint(matrix,x), V))
@@ -169,7 +171,6 @@ def _rotatedBoxLimits( x, y, w, h, angle):
     Y = [x[1] for x in C]
     return min(X), max(X), min(Y), max(Y), C
 
-
 class _DrawTimeResizeable:
     '''Addin class to provide the horribleness of _drawTimeResize'''
     def _drawTimeResize(self,w,h):
@@ -186,7 +187,6 @@ class _SetKeyWordArgs:
         for key, value in keywords.items():
             setattr(self, key, value)
 
-
 #################################################################
 #
 #    Helper functions for working out bounds
@@ -450,8 +450,10 @@ class Group(Shape):
         ''' return a fully expanded object'''
         from reportlab.graphics.widgetbase import Widget
         obj = Group()
+        if hasattr(self,'__label__'):
+            obj.__label__=self.__label__
         if hasattr(obj,'transform'): obj.transform = self.transform[:]
-        P = self.contents[:]    # pending nodes
+        P = self.getContents()[:]   # pending nodes
         while P:
             n = P.pop(0)
             if isinstance(n, UserNode):
@@ -504,7 +506,6 @@ class Group(Shape):
         """Convenience to help you set transforms"""
         self.transform = mmult(self.transform, scale(sx, sy))
 
-
     def skew(self, kx, ky):
         """Convenience to help you set transforms"""
         self.transform = mmult(mmult(self.transform, skewX(kx)),skewY(ky))
@@ -579,14 +580,23 @@ def _repr(self,I=None):
         return 'EmptyClipPath'
     elif isinstance(self,Shape):
         if I: _addObjImport(self,I)
-        from inspect import getargs
-        args, varargs, varkw = getargs(self.__init__.__func__.__code__)
+        from inspect import getargspec
+        args, varargs, varkw, defaults = getargspec(self.__init__)
+        if defaults:
+            kargs = args[-len(defaults):]
+            del args[-len(defaults):]
+        else:
+            kargs = []
         P = self.getProperties()
         s = self.__class__.__name__+'('
         for n in args[1:]:
             v = P[n]
             del P[n]
             s = s + '%s,' % _repr(v,I)
+        for n in kargs:
+            v = P[n]
+            del P[n]
+            s = s + '%s=%s,' % (n,_repr(v,I))
         for n,v in P.items():
             v = P[n]
             s = s + '%s=%s,' % (n, _repr(v,I))
@@ -602,10 +612,12 @@ def _renderGroupPy(G,pfx,I,i=0,indent='\t\t'):
     for n in C:
         if isinstance(n, Group):
             npfx = 'v%d' % i
-            i = i + 1
-            s = s + '%s%s=%s._nn(Group())\n' % (indent,npfx,pfx)
+            i += 1
+            l = getattr(n,'__label__','')
+            if l: l='#'+l
+            s = s + '%s%s=%s._nn(Group())%s\n' % (indent,npfx,pfx,l)
             s = s + _renderGroupPy(n,npfx,I,i,indent)
-            i = i - 1
+            i -= 1
         else:
             s = s + '%s%s.add(%s)\n' % (indent,pfx,_repr(n,I))
     return s
@@ -639,9 +651,16 @@ class Drawing(Group, Flowable):
         #AR temporary hack to track back up.
         #fontName = AttrMapValue(isStringOrNone),
         renderScale = AttrMapValue(isNumber,desc="Global scaling for rendering"),
+        initialFontName = AttrMapValue(isStringOrNone,desc="override the STATE_DEFAULTS value for fontName"),
+        initialFontSize = AttrMapValue(isNumberOrNone,desc="override the STATE_DEFAULTS value for fontSize"),
         )
 
-    _attrMap = AttrMap(BASE=Group)
+    _attrMap = AttrMap(BASE=Group,
+            formats = AttrMapValue(SequenceOf(
+                OneOf('pdf','gif','png','tif','jpg','tiff','pct','pict',
+                        'bmp','tiffp','tiffl','tiff1','eps','svg','ps','py'),
+                lo=1,emptyOK=0), desc='One or more plot modes'),
+            )
     _attrMap.update(_xtraAttrMap)
 
     def __init__(self, width=400, height=200, *nodes, **keywords):
@@ -654,7 +673,10 @@ class Drawing(Group, Flowable):
         self.renderScale = 1.0
 
     def _renderPy(self):
-        I = {'reportlab.graphics.shapes': ['_DrawingEditorMixin','Drawing','Group']}
+        I = {
+                'reportlab.graphics.shapes': ['_DrawingEditorMixin','Drawing','Group'],
+                'reportlab.lib.colors': ['Color','CMYKColor','PCMYKColor'],
+            }
         G = _renderGroupPy(self._explode(),'self',I)
         n = 'ExplodedDrawing_' + self.__class__.__name__
         s = '#Autogenerated by ReportLab guiedit do not edit\n'
@@ -808,7 +830,8 @@ class Drawing(Group, Flowable):
         if 'py' in plotMode:
             filename = fnroot+'.py'
             if verbose: print(genFmt % ('py',filename))
-            open(filename,'wb').write(asBytes(self._renderPy().replace('\n',os.linesep)))
+            with open(filename,'wb') as f:
+                f.write(asBytes(self._renderPy().replace('\n',os.linesep)))
             ext = ext +  '/.py'
 
         logger.warnOnce.enabled, logger.infoOnce.enabled = _saved
@@ -909,7 +932,6 @@ class LineShape(Shape):
         self.strokeOpacity = None
         self.setProperties(kw)
 
-
 class Line(LineShape):
     _attrMap = AttrMap(BASE=LineShape,
         x1 = AttrMapValue(isNumber,desc=""),
@@ -937,6 +959,7 @@ class SolidShape(LineShape):
         fillOpacity = AttrMapValue(isOpacity,desc="the level of transparency of the color, any real number between 0 and 1"),
         fillOverprint = AttrMapValue(isBoolean,desc='Turn on fill overprinting'),
         overprintMask = AttrMapValue(isBoolean,desc='overprinting for ordinary CMYK',advancedUsage=1),
+        fillMode = AttrMapValue(OneOf(FILL_EVEN_ODD,FILL_NON_ZERO)),
         )
 
     def __init__(self, kw):
@@ -946,31 +969,50 @@ class SolidShape(LineShape):
         #the above settings
         LineShape.__init__(self, kw)
 
-
 # path operator  constants
 _MOVETO, _LINETO, _CURVETO, _CLOSEPATH = list(range(4))
 _PATH_OP_ARG_COUNT = (2, 2, 6, 0)  # [moveTo, lineTo, curveTo, closePath]
 _PATH_OP_NAMES=['moveTo','lineTo','curveTo','closePath']
 
-def _renderPath(path, drawFuncs):
+def _renderPath(path,drawFuncs,countOnly=False,forceClose=False):
     """Helper function for renderers."""
     # this could be a method of Path...
     points = path.points
     i = 0
     hadClosePath = 0
     hadMoveTo = 0
+    active = not countOnly
     for op in path.operators:
+        if op == _MOVETO:
+            if forceClose:
+                if hadMoveTo and pop!=_CLOSEPATH:
+                    hadClosePath += 1
+                    if active:
+                        drawFuncs[_CLOSEPATH]()
+            hadMoveTo += 1
         nArgs = _PATH_OP_ARG_COUNT[op]
-        func = drawFuncs[op]
         j = i + nArgs
-        func(*points[i:j])
+        drawFuncs[op](*points[i:j])
         i = j
         if op == _CLOSEPATH:
-            hadClosePath = hadClosePath + 1
-        if op == _MOVETO:
-            hadMoveTo += 1
+            hadClosePath += 1
+        pop = op
+    if forceClose and hadMoveTo and pop!=_CLOSEPATH:
+        hadClosePath += 1
+        if active:
+            drawFuncs[_CLOSEPATH]()
     return hadMoveTo == hadClosePath
 
+_fillModeMap = {
+        None: None,
+        FILL_NON_ZERO: FILL_NON_ZERO,
+        'non-zero': FILL_NON_ZERO,
+        'nonzero': FILL_NON_ZERO,
+        FILL_EVEN_ODD: FILL_EVEN_ODD,
+        'even-odd': FILL_EVEN_ODD,
+        'evenodd': FILL_EVEN_ODD,
+        }
+
 class Path(SolidShape):
     """Path, made up of straight lines and bezier curves."""
 
@@ -978,9 +1020,11 @@ class Path(SolidShape):
         points = AttrMapValue(isListOfNumbers),
         operators = AttrMapValue(isListOfNumbers),
         isClipPath = AttrMapValue(isBoolean),
+        autoclose = AttrMapValue(NoneOr(OneOf('svg','pdf'))),
+        fillMode = AttrMapValue(OneOf(FILL_EVEN_ODD,FILL_NON_ZERO)),
         )
 
-    def __init__(self, points=None, operators=None, isClipPath=0, **kw):
+    def __init__(self, points=None, operators=None, isClipPath=0, autoclose=None, fillMode=None, **kw):
         SolidShape.__init__(self, kw)
         if points is None:
             points = []
@@ -990,6 +1034,7 @@ class Path(SolidShape):
         self.points = points
         self.operators = operators
         self.isClipPath = isClipPath
+        self.autoclose=autoclose
 
     def copy(self):
         new = self.__class__(self.points[:], self.operators[:])
@@ -1129,7 +1174,6 @@ class Rect(SolidShape):
     def getBounds(self):
         return (self.x, self.y, self.x + self.width, self.y + self.height)
 
-
 class Image(SolidShape):
     """Bitmap image."""
 
@@ -1216,11 +1260,13 @@ class Wedge(SolidShape):
         yradius = AttrMapValue(isNumberOrNone),
         radius1 = AttrMapValue(isNumberOrNone),
         yradius1 = AttrMapValue(isNumberOrNone),
+        annular = AttrMapValue(isBoolean,desc='treat as annular ring'),
         )
 
     degreedelta = 1 # jump every 1 degrees
 
-    def __init__(self, centerx, centery, radius, startangledegrees, endangledegrees, yradius=None, **kw):
+    def __init__(self, centerx, centery, radius, startangledegrees, endangledegrees, yradius=None,
+            annular=False, **kw):
         SolidShape.__init__(self, kw)
         while endangledegrees<startangledegrees:
             endangledegrees = endangledegrees+360
@@ -1228,6 +1274,7 @@ class Wedge(SolidShape):
         self.centerx, self.centery, self.radius, self.startangledegrees, self.endangledegrees = \
             centerx, centery, radius, startangledegrees, endangledegrees
         self.yradius = yradius
+        self.annular = annular
 
     def _xtraRadii(self):
         yradius = getattr(self, 'yradius', None)
@@ -1269,7 +1316,7 @@ class Wedge(SolidShape):
         CA = []
         CAA = CA.append
         a = points.append
-        for angle in range(n):
+        for angle in xrange(n):
             angle = startangle+angle*radiansdelta
             CAA((cos(angle),sin(angle)))
         for c,s in CA:
@@ -1282,7 +1329,19 @@ class Wedge(SolidShape):
             for c,s in CA:
                 a(centerx+radius1*c)
                 a(centery+yradius1*s)
-        return Polygon(points)
+        if self.annular:
+            P = Path(fillMode=self.fillMode)
+            P.moveTo(points[0],points[1])
+            for x in xrange(2,2*n,2):
+                P.lineTo(points[x],points[x+1])
+            P.closePath()
+            P.moveTo(points[2*n],points[2*n+1])
+            for x in xrange(2*n+2,4*n,2):
+                P.lineTo(points[x],points[x+1])
+            P.closePath()
+            return P
+        else:
+            return Polygon(points)
 
     def copy(self):
         new = self.__class__(self.centerx,
@@ -1349,7 +1408,7 @@ class PolyLine(LineShape):
     def getBounds(self):
         return getPointsBounds(self.points)
 
-def numericXShift(tA,text,w,fontName,fontSize,encoding=None,pivotCharacter='.'):
+def numericXShift(tA,text,w,fontName,fontSize,encoding=None,pivotCharacter=decimalSymbol):
     dp = getattr(tA,'_dp',pivotCharacter)
     i = text.rfind(dp)
     if i>=0:
@@ -1422,7 +1481,6 @@ class UserNode(_DrawTimeResizeable):
 
         raise NotImplementedError("this method must be redefined by the user/programmer")
 
-
 def test():
     r = Rect(10,10,200,50)
     import pprint
@@ -1440,6 +1498,5 @@ def test():
     w('verifying...')
     r.verify()
 
-
 if __name__=='__main__':
     test()
