@@ -1,7 +1,7 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/lib/abag.py
-__version__=''' $Id: abag.py 3959 2012-09-27 14:39:39Z robin $ '''
+__version__='3.3.0'
 __doc__='''Data structure to hold a collection of attributes, used by styles.'''
 class ABag:
     """
@@ -19,18 +19,18 @@ class ABag:
         self.__dict__.update(attr)
 
     def clone(self,**attr):
-        n = ABag(**self.__dict__)
+        n = self.__class__(**self.__dict__)
         if attr: n.__dict__.update(attr)
         return n
 
     def __repr__(self):
         D = self.__dict__
-        K = D.keys()
+        K = list(D.keys())
         K.sort()
         return '%s(%s)' % (self.__class__.__name__,', '.join(['%s=%r' % (k,D[k]) for k in K]))
 
 if __name__=="__main__":
     AB = ABag(a=1, c="hello")
     CD = AB.clone()
-    print AB
-    print CD
+    print(AB)
+    print(CD)
