@@ -1,6 +1,6 @@
-#copyright ReportLab Europe Limited. 2000-2012
+#copyright ReportLab Europe Limited. 2000-2016
 #see license.txt for license details
-__version__=''' $Id$ '''
+__version__='3.3.0'
 
 """helpers for pdf encryption/decryption"""
 import sys, os, tempfile
@@ -384,7 +384,7 @@ def encryptPdfInMemory(inputPDF,
         from rlextra.pageCatcher.pageCatcher import storeFormsInMemory, restoreFormsInMemory
     except ImportError:
         raise ImportError('''reportlab.lib.pdfencrypt.encryptPdfInMemory failed because rlextra cannot be imported.
-See http://developer.reportlab.com''')
+See https://www.reportlab.com/downloads''')
 
     (bboxInfo, pickledForms) = storeFormsInMemory(inputPDF, all=1, BBoxes=1)
     names = list(bboxInfo.keys())
@@ -405,13 +405,12 @@ See http://developer.reportlab.com''')
 
     formNames = restoreFormsInMemory(pickledForms, canv)
     for formName in formNames:
-        #need to extract page size in future
+        canv.setPageSize(bboxInfo[formName][2:])
         canv.doForm(formName)
         canv.showPage()
     canv.save()
     return buf.getvalue()
 
-
 def encryptPdfOnDisk(inputFileName, outputFileName,
                   userPassword, ownerPassword=None,
                   canPrint=1, canModify=1, canCopy=1, canAnnotate=1,
@@ -592,7 +591,7 @@ See PdfEncryptIntro.pdf for more information.
             print("wrote output file '%s'(%s bytes)\n  owner password is '%s'\n  user password is '%s'" % (SAVEFILE, filesize, OWNER, USER))
 
         if len(argv)>0:
-            raise valueError("\nUnrecognised arguments : %s\nknown arguments are:\n%s" % (str(argv)[1:-1], known_modes))
+            raise ValueError("\nUnrecognised arguments : %s\nknown arguments are:\n%s" % (str(argv)[1:-1], known_modes))
     else:
         print(usage)
 
