@@ -1,7 +1,7 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/pdfbase/pdfdoc.py
-__version__=''' $Id$ '''
+__version__='3.3.0'
 __doc__="""
 The module pdfdoc.py handles the 'outer structure' of PDF documents, ensuring that
 all objects are properly cross-referenced and indexed to the nearest byte.  The
@@ -14,11 +14,11 @@ The classes within this generally mirror structures in the PDF file
 and are not part of any public interface.  Instead, canvas and font
 classes are made available elsewhere for users to manipulate.
 """
-import types, binascii, codecs
+import types, binascii, codecs, time
 from collections import OrderedDict
 from reportlab.pdfbase import pdfutils
 from reportlab import rl_config
-from reportlab.lib.utils import import_zlib, open_for_read, makeFileName, isSeq, isBytes, isUnicode, _digester, isStr, bytestr, isPy3
+from reportlab.lib.utils import import_zlib, open_for_read, makeFileName, isSeq, isBytes, isUnicode, _digester, isStr, bytestr, isPy3, annotateException
 from reportlab.lib.rl_accel import escapePDF, fp_str, asciiBase85Encode, asciiBase85Decode
 from reportlab.pdfbase import pdfmetrics
 from hashlib import md5
@@ -79,7 +79,7 @@ def format(element, document, toplevel=0):
         else:
             f = element.format(document)
             if not rl_config.invariant and rl_config.pdfComments and hasattr(element, __Comment__):
-                f = pdfdocEnc("%% %s\r\n" % element.__Comment__)+f
+                f = pdfdocEnc("%% %s\n" % element.__Comment__)+f
             return f
     elif type(element) in (float, int):
         #use a controlled number formatting routine
@@ -202,12 +202,14 @@ class PDFDocument(PDFObject):
             return self._ID
         digest = self.signature.digest()
         doc = DummyDoc()
-        IDs = PDFString(digest,enc='raw').format(doc)
-        self._ID = (b'\r\n % ReportLab generated PDF document -- digest (http://www.reportlab.com)\r\n ['
-                        +IDs+b' '+IDs+b']\r\n')
+        IDs = PDFText(digest,enc='raw').format(doc)
+        self._ID = (b'\n['+IDs+IDs+b']\n% ReportLab generated PDF document -- digest (http://www.reportlab.com)\n')
         return self._ID
 
     def SaveToFile(self, filename, canvas):
+        if getattr(self,'_savedToFile',False):
+            raise RunTimeError("class %s instances can only be saved once" % self.__class__.__name__)
+        self._savedToFile = True
         if hasattr(getattr(filename, "write",None),'__call__'):
             myfile = 0
             f = filename
@@ -269,7 +271,6 @@ class PDFDocument(PDFObject):
                 if fontObj._dynamicFont:
                     raise PDFError("getInternalFontName(%s) called for a dynamic font" % repr(psfontname))
                 fontObj.addObjects(self)
-                #self.addFont(fontObj)
                 return fm[psfontname]
             except KeyError:
                 raise PDFError("Font %s not known!" % repr(psfontname))
@@ -394,8 +395,8 @@ class PDFDocument(PDFObject):
         self.encrypt.prepare(self)
         cat = self.Catalog
         info = self.info
-        self.Reference(self.Catalog)
-        self.Reference(self.info)
+        self.Reference(cat)
+        self.Reference(info)
         # register the encryption dictionary if present
         encryptref = None
         encryptinfo = self.encrypt.info()
@@ -409,31 +410,27 @@ class PDFDocument(PDFObject):
         idToOb = self.idToObject
         idToOf = self.idToOffset
         ### note that new entries may be "appended" DURING FORMATTING
-        done = None
         # __accum__ allows objects to know where they are in the file etc etc
         self.__accum__ = File = PDFFile(self._pdfVersion) # output collector
-        while done is None:
+        while True:
             counter += 1 # do next object...
-            if counter in numbertoid:
-                id = numbertoid[counter]
-                #printidToOb
-                obj = idToOb[id]
-                IO = PDFIndirectObject(id, obj)
-                # register object number and version
-                #encrypt.register(id,
-                IOf = IO.format(self)
-                # add a comment to the PDF output
-                if not rl_config.invariant and rl_config.pdfComments:
-                    try:
-                        classname = obj.__class__.__name__
-                    except:
-                        classname = ascii(obj)
-                    File.add("%% %s: class %s \r\n" % (ascii(id), classname[:50]))
-                offset = File.add(IOf)
-                idToOf[id] = offset
-                ids.append(id)
-            else:
-                done = 1
+            if counter not in numbertoid: break
+            oid = numbertoid[counter]
+            #printidToOb
+            obj = idToOb[oid]
+            IO = PDFIndirectObject(oid, obj)
+            # register object number and version
+            IOf = IO.format(self)
+            # add a comment to the PDF output
+            if not rl_config.invariant and rl_config.pdfComments:
+                try:
+                    classname = obj.__class__.__name__
+                except:
+                    classname = ascii(obj)
+                File.add("%% %s: class %s \n" % (ascii(oid), classname[:50]))
+            offset = File.add(IOf)
+            idToOf[oid] = offset
+            ids.append(oid)
         del self.__accum__
         # sanity checks (must happen AFTER formatting)
         lno = len(numbertoid)
@@ -538,12 +535,13 @@ PDFfalse = "false"
 PDFnull = "null"
 
 class PDFText(PDFObject):
-    def __init__(self, t):
+    def __init__(self, t, enc='utf-8'):
         self.t = t
+        self.enc = enc
     def format(self, document):
         t = self.t
         if isUnicode(t):
-            t = t.encode('utf-8')
+            t = t.encode(self.enc)
         result = binascii.hexlify(document.encrypt.encode(t))
         return b"<" + result + b">"
     def __str__(self):
@@ -580,7 +578,7 @@ class PDFString(PDFObject):
         '''s can be unicode/utf8 or a PDFString
         if escape is true then the output will be passed through escape
         if enc is raw then the string will be left alone
-        if enc is auto we'll try and automatically adapt to utf_16_be if the
+        if enc is auto we'll try and automatically adapt to utf_16_be/utf_16_le if the
         effective string is not entirely in pdfdoc
         '''
         if isinstance(s,PDFString):
@@ -594,10 +592,15 @@ class PDFString(PDFObject):
     def format(self, document):
         s = self.s
         enc = getattr(self,'enc','auto')
-        if (isBytes(s)):
+        if isBytes(s):
             if enc is 'auto':
                 try:
-                    u = s.decode(s.startswith(codecs.BOM_UTF16_BE) and 'utf16' or 'utf8')
+                    if s.startswith(codecs.BOM_UTF16_BE):
+                        u = s.decode('utf_16_be')
+                    elif s.startswith(codecs.BOM_UTF16_LE):
+                        u = s.decode('utf_16_le')
+                    else:
+                        u = s.decode('utf8')
                     if _checkPdfdoc(u):
                         s = u.encode('pdfdoc')
                     else:
@@ -665,7 +668,7 @@ class PDFDictionary(PDFObject):
         return a in self.dict
     def Reference(self, name, document):
         self.dict[name] = document.Reference(self.dict[name])
-    def format(self, document,IND=b'\r\n '):
+    def format(self, document,IND=b'\n '):
         dict = self.dict
         try:
             keys = list(dict.keys())
@@ -674,15 +677,15 @@ class PDFDictionary(PDFObject):
             raise
         if not isinstance(dict,OrderedDict): keys.sort()
         L = [(format(PDFName(k),document)+b" "+format(dict[k],document)) for k in keys]
-        if self.multiline and rl_config.pdfMultiLine:
+        if (self.multiline and rl_config.pdfMultiLine) or self.multiline=='forced':
             L = IND.join(L)
         else:
             # break up every 6 elements anyway
             t=L.insert
             for i in reversed(range(6, len(L), 6)):
-                t(i,b'\r\n ')
+                t(i,b'\n ')
             L = b" ".join(L)
-        return b'<< '+L+b' >>'
+        return b'<<\n'+L+b'\n>>'
 
     def copy(self):
         return PDFDictionary(self.dict)
@@ -811,14 +814,13 @@ class PDFStream(PDFObject):
         fc = format(content, document)
         dictionary["Length"] = len(content)
         fd = format(dictionary, document)
-        return fd+b'\r\nstream\r\n'+fc+b'endstream\r\n'
+        return fd+b'\nstream\n'+fc+b'endstream\n'
 
 def teststream(content=None):
     #content = "" # test
     if content is None:
         content = teststreamcontent
-    content = content.strip()
-    content = content.replace("\n", '\n\r') + '\n\r'
+    content = content.strip() + '\n'
     S = PDFStream(content = content,
                     filters=rl_config.useA85 and [PDFBase85Encode,PDFZCompress] or [PDFZCompress])
     # nothing else needed...
@@ -837,9 +839,9 @@ class PDFArray(PDFObject):
     def References(self, document):
         """make all objects in sequence references"""
         self.sequence = list(map(document.Reference, self.sequence))
-    def format(self, document, IND=b'\r\n '):
+    def format(self, document, IND=b'\n '):
         L = [format(e, document) for e in self.sequence]
-        if self.multiline and rl_config.pdfMultiLine:
+        if (self.multiline and rl_config.pdfMultiLine) or self.multiline=='forced':
             L = IND.join(L)
         else:
             n=len(L)
@@ -847,7 +849,7 @@ class PDFArray(PDFObject):
                 # break up every 10 elements anyway
                 t=L.insert
                 for i in reversed(range(10, n, 10)):
-                    t(i,b'\r\n ')
+                    t(i,b'\n ')
                 L = b' '.join(L)
             else:
                 L = b' '.join(L)
@@ -867,9 +869,9 @@ class PDFIndirectObject(PDFObject):
         # set encryption parameters
         document.encrypt.register(n, v)
         fcontent = format(self.content, document, toplevel=1)   # yes this is at top level
-        return (pdfdocEnc("%s %s obj\r\n"%(n,v))
-                +fcontent+ (b'' if fcontent.endswith(b'\r\n') else b'\r\n')
-                +b'endobj\r\n')
+        return (pdfdocEnc("%s %s obj\n"%(n,v))
+                +fcontent+ (b'' if fcontent.endswith(b'\n') else b'\n')
+                +b'endobj\n')
 
 class PDFObjectReference(PDFObject):
     def __init__(self, name):
@@ -893,7 +895,7 @@ class PDFFile(PDFObject):
         # any other encoding, and we'll be able to tell if something
         # has run our PDF files through a dodgy Unicode conversion.
         self.add((pdfdocEnc("%%PDF-%s.%s" % pdfVersion) +
-            b'\r\n%\223\214\213\236 ReportLab Generated PDF document http://www.reportlab.com\r\n'
+            b'\n%\223\214\213\236 ReportLab Generated PDF document http://www.reportlab.com\n'
             ))
 
     def closeOrReset(self):
@@ -910,8 +912,6 @@ class PDFFile(PDFObject):
     def format(self, document):
         return b''.join(self.strings)
 
-XREFFMT = '%0.10d %0.5d n'
-
 class PDFCrossReferenceSubsection(PDFObject):
     def __init__(self, firstentrynumber, idsequence):
         self.firstentrynumber = firstentrynumber
@@ -927,7 +927,7 @@ class PDFCrossReferenceSubsection(PDFObject):
         if firstentrynumber==0:
             taken[0] = "standard free entry"
             nentries = nentries+1
-            entries.insert(0, "0000000000 65535 f")
+            entries.insert(0, "0000000000 65535 f ")
         idToNV = document.idToObjectNumberAndVersion
         idToOffset = document.idToOffset
         lastentrynumber = firstentrynumber+nentries-1
@@ -941,13 +941,13 @@ class PDFCrossReferenceSubsection(PDFObject):
             rnum = num-firstentrynumber
             taken[num] = id
             offset = idToOffset[id]
-            entries[num] = XREFFMT % (offset, version)
+            entries[num] = '%0.10d %0.5d n ' % (offset, version)
         # now add the initial line
         firstline = "%s %s" % (firstentrynumber, nentries)
         entries.insert(0, firstline)
-        # make sure it ends with \r\n
+        # make sure it ends with \n
         entries.append("")
-        return pdfdocEnc('\r\n'.join(entries))
+        return pdfdocEnc('\n'.join(entries))
 
 class PDFCrossReferenceTable(PDFObject):
     def __init__(self):
@@ -959,7 +959,7 @@ class PDFCrossReferenceTable(PDFObject):
         sections = self.sections
         if not sections:
             raise ValueError("no crossref sections")
-        L = [b"xref\r\n"]
+        L = [b"xref\n"]
         for s in self.sections:
             fs = format(s, document)
             L.append(fs)
@@ -975,14 +975,15 @@ class PDFTrailer(PDFObject):
                       ("Info", Info), ("ID", ID), ("Encrypt", Encrypt)]:
             if v is not None:
                 dict[n] = v
+        dict.multiline='forced'
     def format(self, document):
-        fdict = format(self.dict, document)
+        fdict = self.dict.format(document,IND=b'\n')
         return b''.join([
-                b'trailer\r\n',
+                b'trailer\n',
                 fdict,
-                b'\r\nstartxref\r\n',
+                b'\nstartxref\n',
                 pdfdocEnc(str(self.startxref)),
-                b'\r\n%%EOF\r\n',
+                b'\n%%EOF\n',
                 ]
                 )
 
@@ -997,31 +998,26 @@ class PDFCatalog(PDFObject):
     # to override, set as attributes
     __Defaults__ = {"Type": PDFName("Catalog"),
                 "PageMode": PDFName("UseNone"),
+                "Lang": None,
                 }
     __NoDefault__ = """
         Dests Outlines Pages Threads AcroForm Names OpenAction PageMode URI
         ViewerPreferences PageLabels PageLayout JavaScript StructTreeRoot SpiderInfo""".split()
-    __Refs__ = __NoDefault__ # make these all into references, if present
+    __Refs__ = __NoDefault__
 
     def format(self, document):
         self.check_format(document)
         defaults = self.__Defaults__
         Refs = self.__Refs__
         D = {}
-        for k in defaults.keys():
-            default = defaults[k]
-            v = None
-            if hasattr(self, k) and getattr(self,k) is not None:
-                v = getattr(self, k)
-            elif default is not None:
-                v = default
+        for k,v in defaults.items():
+            v = getattr(self,k,v)
             if v is not None:
                 D[k] = v
         for k in self.__NoDefault__:
-            if hasattr(self, k):
-                v = getattr(self,k)
-                if v is not None:
-                    D[k] = v
+            v = getattr(self,k,None)
+            if v is not None:
+                D[k] = v
         # force objects to be references where required
         for k in Refs:
             if k in D:
@@ -1108,7 +1104,7 @@ class PDFPage(PDFCatalog):
         if self.Override_default_compilation:
             raise ValueError("overridden! must set stream explicitly")
         if isSeq(code):
-            code = '\r\n'.join(code)+'\r\n'
+            code = '\n'.join(code)+'\n'
         self.stream = code
 
     def setPageTransition(self, tranDict):
@@ -1159,6 +1155,8 @@ class PDFPage(PDFCatalog):
             self.Parent = document.Reference(pages)
 
 #this code contributed by  Christian Jacobs <cljacobsen@gmail.com>
+class DuplicatePageLabelPage(Exception):
+    pass
 class PDFPageLabels(PDFCatalog):
     __comment__ = None
     __RefOnly__ = 0
@@ -1186,7 +1184,11 @@ class PDFPageLabels(PDFCatalog):
         self.labels.append((page, label))
 
     def format(self, document):
-        self.labels.sort()
+        try:
+            self.labels.sort()
+        except DuplicatePageLabelPage:
+            tmp = sorted([x[0] for x in self.labels])
+            annotateException('\n\n!!!!! Duplicate PageLabel seen for pages %r' % list(set([x for x in tmp if tmp.count(x)>1])))
         labels = []
         for page, label in self.labels:
             labels.append(page)
@@ -1263,6 +1265,11 @@ class PDFPageLabel(PDFCatalog):
             self.S = PDFName(style)
         if start: self.St = PDFnumber(start)
         if prefix: self.P = PDFString(prefix)
+
+    def __lt__(self,oth):
+        if rl_config.errorOnDuplicatePageLabelPage:
+            raise DuplicatePageLabelPage()
+        return False
 #ends code contributed by  Christian Jacobs <cljacobsen@gmail.com>
 
 def testpage(document):
@@ -1288,7 +1295,7 @@ DUMMYOUTLINE = """
 
 class PDFOutlines0(PDFObject):
     __Comment__ = "TEST OUTLINE!"
-    text = DUMMYOUTLINE.replace("\n", '\r\n')
+    text = DUMMYOUTLINE.replace("\n", '\n')
     __RefOnly__ = 1
     def format(self, document):
         return pdfdocEnc(self.text)
@@ -1532,6 +1539,7 @@ class PDFInfo(PDFObject):
 
     def __init__(self):
         self.invariant = rl_config.invariant
+        self.trapped = 'False'  #could be 'True' or 'Unknown'
 
     def digest(self, md5object):
         # add self information to signature
@@ -1542,11 +1550,12 @@ class PDFInfo(PDFObject):
         D = {}
         D["Title"] = PDFString(self.title)
         D["Author"] = PDFString(self.author)
-        D["CreationDate"] = PDFDate(invariant=self.invariant,dateFormatter=self._dateFormatter)
+        D['ModDate'] = D["CreationDate"] = PDFDate(invariant=self.invariant,dateFormatter=self._dateFormatter)
         D["Producer"] = PDFString(self.producer)
         D["Creator"] = PDFString(self.creator)
         D["Subject"] = PDFString(self.subject)
         D["Keywords"] = PDFString(self.keywords)
+        D["Trapped"] = PDFName(self.trapped)
 
         PD = PDFDictionary(D)
         return PD.format(document)
@@ -1723,7 +1732,6 @@ _NOWT=None
 def _getTimeStamp():
     global _NOWT
     if not _NOWT:
-        import time
         _NOWT = time.time()
     return _NOWT
 
@@ -1735,11 +1743,9 @@ class PDFDate(PDFObject):
             self.dhh = 0
             self.dmm = 0
         else:
-            import time
             now = tuple(time.localtime(_getTimeStamp())[:6])
-            from time import timezone
-            self.dhh = int(timezone / (3600.0))
-            self.dmm = (timezone % 3600) % 60
+            self.dhh = int(time.timezone / (3600.0))
+            self.dmm = (time.timezone % 3600) % 60
         self.date = now[:6]
         self.dateFormatter = dateFormatter
 
@@ -1773,9 +1779,9 @@ class Destination(PDFObject):
         self.fmt = self.page = None
     def format(self, document):
         f = self.fmt
-        if f is None: raise ValueError("format not resolved %s" % self.name)
+        if f is None: raise ValueError("format not resolved, probably missing URL scheme or undefined destination target for '%s'" % self.name)
         p = self.page
-        if p is None: raise ValueError("Page reference unbound %s" % self.name)
+        if p is None: raise ValueError("Page not bound, probably missing URL scheme or undefined destination target for '%s'" % self.name)
         f.page = p
         return f.format(document)
     def xyz(self, left, top, zoom):  # see pdfspec mar 11 99 pp184+
@@ -1994,7 +2000,7 @@ class PDFFormXObject(PDFObject):
 
     def setStreamList(self, data):
         if isSeq(data):
-            data = '\r\n'.join(data)
+            data = '\n'.join(data)
         self.stream = pdfdocEnc(data)
 
     def BBoxList(self):
@@ -2101,7 +2107,10 @@ class PDFImageXObject(PDFObject):
 
     def loadImageFromA85(self,source):
         IMG=[]
-        imagedata = [s.strip() for s in pdfutils.makeA85Image(source,IMG=IMG)]
+        imagedata = pdfutils.makeA85Image(source,IMG=IMG,detectJpeg=True)
+        if not imagedata:
+            return self.loadImageFromSRC(IMG[0])
+        imagedata = [s.strip() for s in imagedata]
         words = imagedata[1].split()
         self.width, self.height = (int(words[1]),int(words[3]))
         self.colorSpace = {'/RGB':'DeviceRGB', '/G':'DeviceGray', '/CMYK':'DeviceCMYK'}[words[7]]
@@ -2139,7 +2148,9 @@ class PDFImageXObject(PDFObject):
 
     def loadImageFromRaw(self,source):
         IMG=[]
-        imagedata = pdfutils.makeRawImage(source,IMG=IMG)
+        imagedata = pdfutils.makeRawImage(source,IMG=IMG,detectJpeg=True)
+        if not imagedata:
+            return self.loadImageFromSRC(IMG[0])
         words = imagedata[1].split()
         self.width = int(words[1])
         self.height = int(words[3])
