@@ -1,4 +1,4 @@
-#Copyright ReportLab Europe Ltd. 2000-2016
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 '''default settings for reportlab
 
@@ -51,7 +51,9 @@ CMapSearchPath
 baseUnderlineProportion
 decimalSymbol
 errorOnDuplicatePageLabelPage
-autoGenerateMissingTTFName'''.split())
+autoGenerateMissingTTFName
+allowTTFSubsetting
+spaceShrinkage'''.split())
 
 allowTableBoundsErrors =    1 # set to 0 to die on too large elements in tables in debug (recommend 1 for production use)
 shapeChecking =             1
@@ -73,7 +75,7 @@ eps_ttf_embed=              1                       #set to False to disable
 eps_ttf_embed_uid=          0                       #set to 1 to enable
 overlapAttachedSpace=       1                       #if set non false then adajacent flowable space after
                                                     #and space before are merged (max space is used).
-longTableOptimize =         1                       #default don't use Henning von Bargen's long table optimizations
+longTableOptimize =         1                       #default do use Henning von Bargen's long table optimizations
 autoConvertEncoding  =      0                       #convert internally as needed (experimental)
 _FUZZ=                      1e-6                    #fuzz for layout arithmetic
 wrapA85=                    0                       #set to 1 to get old wrapped line behaviour
@@ -107,13 +109,22 @@ decimalSymbol=              '.'                     #what we use to align floats
 errorOnDuplicatePageLabelPage= 0                    #if True will cause repeated PageLabel page numbers to raise an error.
 autoGenerateMissingTTFName=0                        #if true we try to auto generate any missing TTF font name
 
+allowTTFSubsetting=         []                      #list of font file names that will be subsetted even when they
+                                                    #have the no subsetting flag set. These should be fonts for which
+                                                    #the user has explicit permission from the rights holder(s). 
+                                                    #This flag could already be overcome by hacking the code.
+                                                    #ReportLab takes no responsibility for the use of this setting.
+
+spaceShrinkage=0.05                                 #allowable space shrinkage to make lines fit
+
+
 # places to look for T1Font information
 T1SearchPath =  (
                 'c:/Program Files/Adobe/Acrobat 9.0/Resource/Font', 
                 'c:/Program Files/Adobe/Acrobat 8.0/Resource/Font', 
                 'c:/Program Files/Adobe/Acrobat 7.0/Resource/Font', 
                 'c:/Program Files/Adobe/Acrobat 6.0/Resource/Font', #Win32, Acrobat 6
-                'c:/Program Files/Adobe/Acrobat 5.0/Resource/Font',     #Win32, Acrobat 5
+                'c:/Program Files/Adobe/Acrobat 5.0/Resource/Font', #Win32, Acrobat 5
                 'c:/Program Files/Adobe/Acrobat 4.0/Resource/Font', #Win32, Acrobat 4
                 '%(disk)s/Applications/Python %(sys_version)s/reportlab/fonts', #Mac?
                 '/usr/lib/Acrobat9/Resource/Font',      #Linux, Acrobat 5?
