@@ -1,5 +1,5 @@
 #!/bin/env python
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/tools/docco/rl_doc_utils.py
 __version__='3.3.0'
@@ -135,11 +135,14 @@ def bullet(text):
     P = Paragraph(text, BU)
     getStory().append(P)
 
-def eg(text,before=0.1,after=0):
+def eg(text,before=0.1,after=0,klass=PythonPreformatted):
     space(before)
-    disc(text, klass=PythonPreformatted, style=exampletextstyle)
+    disc(text, klass=klass, style=exampletextstyle)
     space(after)
 
+def npeg(text,before=0.1,after=0):
+    eg(text,before=before,after=after,klass=XPreformatted)
+
 def space(inches=1./6):
     if inches: getStory().append(Spacer(0,inches*inch))
 
